@@ -26,167 +26,7 @@
 #include "bitmap.h"
 
 
-namespace {
-
-void enlarge_2( std::vector< std::vector< char > > & data ) throw()
-  {
-  const int height = data.size(), width = data[0].size();
-  std::vector< std::vector< char > > new_data( 2 * height );
-
-  for( unsigned int row = 0; row < new_data.size(); ++row )
-    new_data[row].resize( 2 * width, false );
-
-  for( int row = 0; row < height; ++row )
-    {
-    const std::vector< char > & datarow = data[row];
-    std::vector< char > & new_datarow0 = new_data[2*row];
-    std::vector< char > & new_datarow1 = new_data[2*row+1];
-    for( int col = 0; col < width; ++col )
-      if( datarow[col] )
-        {
-        const bool l = col > 0 && datarow[col-1];
-        const bool t = row > 0 && data[row-1][col];
-        const bool r = col < width - 1 && datarow[col+1];
-        const bool b = row < height - 1 && data[row+1][col];
-        const bool lt = row > 0 && col > 0 && data[row-1][col-1];
-        const bool rt = row > 0 && col < width - 1 && data[row-1][col+1];
-        const bool lb = row < height - 1 && col > 0 && data[row+1][col-1];
-        const bool rb = row < height - 1 && col < width - 1 && data[row+1][col+1];
-
-        if( l || t || lt || ( !rt && !lb ) ) new_datarow0[2*col] = true;
-        if( r || t || rt || ( !lt && !rb ) ) new_datarow0[2*col+1] = true;
-        if( l || b || lb || ( !lt && !rb ) ) new_datarow1[2*col] = true;
-        if( r || b || rb || ( !rt && !lb ) ) new_datarow1[2*col+1] = true;
-        }
-    }
-  data.swap( new_data );
-  }
-
-
-void enlarge_3( std::vector< std::vector< char > > & data ) throw()
-  {
-  const int height = data.size(), width = data[0].size();
-  std::vector< std::vector< char > > new_data( 3 * height );
-
-  for( unsigned int row = 0; row < new_data.size(); ++row )
-    new_data[row].resize( 3 * width, false );
-
-  for( int row = 0; row < height; ++row )
-    {
-    const int row3 = 3 * row;
-    const std::vector< char > & datarow = data[row];
-    std::vector< char > & new_datarow0 = new_data[row3];
-    std::vector< char > & new_datarow1 = new_data[row3+1];
-    std::vector< char > & new_datarow2 = new_data[row3+2];
-    for( int col = 0; col < width; ++col )
-      {
-      const int col3 = 3 * col;
-      const bool l = col > 0 && datarow[col-1];
-      const bool t = row > 0 && data[row-1][col];
-      const bool r = col < width - 1 && datarow[col+1];
-      const bool b = row < height - 1 && data[row+1][col];
-      const bool lt = row > 0 && col > 0 && data[row-1][col-1];
-      const bool rt = row > 0 && col < width - 1 && data[row-1][col+1];
-      const bool lb = row < height - 1 && col > 0 && data[row+1][col-1];
-      const bool rb = row < height - 1 && col < width - 1 && data[row+1][col+1];
-      if( datarow[col] )
-        {
-        if( l || t || lt || ( !rt && !lb ) ) new_datarow0[col3] = true;
-        new_datarow0[col3+1] = true;
-        if( r || t || rt || ( !lt && !rb ) ) new_datarow0[col3+2] = true;
-        new_datarow1[col3] = new_datarow1[col3+1] = new_datarow1[col3+2] = true;
-        if( l || b || lb || ( !lt && !rb ) ) new_datarow2[col3] = true;
-        new_datarow2[col3+1] = true;
-        if( r || b || rb || ( !rt && !lb ) ) new_datarow2[col3+2] = true;
-        }
-      else
-        {
-        if( l && t && lt && ( !rt || !lb ) ) new_datarow0[col3] = true;
-        if( r && t && rt && ( !lt || !rb ) ) new_datarow0[col3+2] = true;
-        if( l && b && lb && ( !lt || !rb ) ) new_datarow2[col3] = true;
-        if( r && b && rb && ( !rt || !lb ) ) new_datarow2[col3+2] = true;
-        }
-      }
-    }
-  data.swap( new_data );
-  }
-
-
-void enlarge_n( std::vector< std::vector< char > > & data, const int n ) throw()
-  {
-  if( n < 2 ) return;
-  const int height = data.size(), width = data[0].size();
-  std::vector< std::vector< char > > new_data;
-  new_data.reserve( n * height );
-
-  for( int row = 0; row < height; ++row )
-    {
-    const std::vector< char > & datarow = data[row];
-    new_data.push_back( std::vector< char >() );
-    for( int col = 0; col < width; ++col )
-      {
-      const char d = datarow[col];
-      for( int i = 0; i < n; ++i ) new_data.back().push_back( d );
-      }
-    for( int i = 1; i < n; ++i ) new_data.push_back( new_data.back() );
-    }
-  data.swap( new_data );
-  }
-
-
-void mirror_left_right( std::vector< std::vector< char > > & data ) throw()
-  {
-  const int height = data.size();
-  for( int row = 0; row < height; ++row )
-    std::reverse( data[row].begin(), data[row].end() );
-  }
-
-
-void mirror_top_bottom( std::vector< std::vector< char > > & data ) throw()
-  {
-  for( int u = 0, d = data.size() - 1; u < d; ++u, --d )
-    data[u].swap( data[d] );
-  }
-
-
-void mirror_diagonal( std::vector< std::vector< char > > & data,
-                      Rectangle & re ) throw()
-  {
-  const int size = std::max( re.height(), re.width() );
-
-  if( re.height() < size )
-    {
-    data.resize( size );
-    for( int row = re.height(); row < size; ++row )
-      data[row].resize( size );
-    }
-  else if( re.width() < size )
-    for( int row = 0; row < re.height(); ++row )
-      data[row].resize( size );
-
-  for( int row = 0; row < size; ++row )
-    {
-    std::vector< char > & datarow = data[row];
-    for( int col = 0; col < row; ++col )
-      {
-      bool b = datarow[col];
-      datarow[col] = data[col][row]; data[col][row] = b;
-      }
-    }
-
-  const int h = re.height(), w = re.width();
-  re.height( w ); re.width( h );
-  if( re.height() < size ) data.resize( re.height() );
-  else if( re.width() < size )
-    for( int row = 0; row < re.height(); ++row )
-      data[row].resize( re.width() );
-  }
-
-} // end namespace
-
-
-// Creates a blank Bitmap
-//
+    // Creates a blank Bitmap
 Bitmap::Bitmap( const int l, const int t, const int r, const int b ) throw()
   : Rectangle( l, t, r, b ), data( height() )
   {
@@ -195,8 +35,7 @@ Bitmap::Bitmap( const int l, const int t, const int r, const int b ) throw()
   }
 
 
-// Creates a Bitmap from part of another Bitmap
-//
+    // Creates a Bitmap from part of another Bitmap
 Bitmap::Bitmap( const Bitmap & source, const Rectangle & re ) throw()
   : Rectangle( re ), data( re.height() )
   {
@@ -614,47 +453,3 @@ bool Bitmap::bottom_hook( int *hdiff ) const throw()
     }
   return false;
   }
-
-
-bool Bitmap::scale( int n, const Rational & th ) throw( Bitmap::Error )
-  {
-  if( n <= -2 )
-    { Bitmap reduced( *this, -n, th ); *this = reduced; return true; }
-  if( n >= 2 )
-    {
-    if( (long long)width() * height() * n > (long long)INT_MAX )
-      throw Error( "scale factor too big. `int' will overflow." );
-    if( n && ( n % 2 ) == 0 ) { enlarge_2( data ); n /= 2; }
-    else if( n && ( n % 3 ) == 0 ) { enlarge_3( data ); n /= 3; }
-    if( n > 1 ) enlarge_n( data, n );
-    Rectangle::height( data.size() );
-    Rectangle::width( data[0].size() );
-    return true;
-    }
-  return false;
-  }
-
-
-void Bitmap::transform( const Transformation & t ) throw()
-  {
-  switch( t.type() )
-    {
-    case Transformation::none:
-      break;
-    case Transformation::rotate90:
-      mirror_diagonal( data, *this ); mirror_top_bottom( data ); break;
-    case Transformation::rotate180:
-      mirror_left_right( data ); mirror_top_bottom( data ); break;
-    case Transformation::rotate270:
-      mirror_diagonal( data, *this ); mirror_left_right( data ); break;
-    case Transformation::mirror_lr:
-      mirror_left_right( data ); break;
-    case Transformation::mirror_tb:
-      mirror_top_bottom( data ); break;
-    case Transformation::mirror_d1:
-      mirror_diagonal( data, *this ); break;
-    case Transformation::mirror_d2:
-      mirror_diagonal( data, *this );
-      mirror_left_right( data ); mirror_top_bottom( data ); break;
-    }
-  }
