@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
@@ -68,11 +69,10 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
     add_guess( code, 0 ); return;
     }
   if( b.height() < 5 || ( b.height() < 8 && b.width() < 6 ) ||
-      ( b.height() > 10 * b.width() ) ||
-      ( b.height() * 5 < b.width() ) ) return;
+      b.height() > 10 * b.width() || 5 * b.height() < b.width() ) return;
 
   code = f.test_CEFIJLlT( charset ); if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_frst();              if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_frst( charbox );     if( code ) { add_guess( code, 0 ); return; }
   code = f.test_G();                 if( code ) { add_guess( code, 0 ); return; }
   code = f.test_c();                 if( code ) { add_guess( code, 0 ); return; }
   if( charset.enabled( Charset::iso_8859_9 ) )
@@ -89,25 +89,40 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
              ( b.seek_top( b.vcenter(), col ) > b.top() ||
                f.hp[col-b.left()] > b.height() / 10 ) );
       if( col > b.hpos( 40 ) && col < b.right() )
-        { only_guess( 0, b.left() ); add_guess( 't', col );
-        add_guess( 'r', b.right() ); return; }
+        {
+        only_guess( 0, b.left() ); add_guess( 't', col );
+        add_guess( 'r', b.right() ); return;
+        }
+      }
+    }
+  if( code == 'N' && b.width() > b.height() && b.top() > charbox.top() )
+    {					// Looks for merged 'rv'
+    const int col = f.hp.iminimum();
+    if( col > f.hp.pos( 40 ) && col < f.hp.pos( 50 ) )
+      {
+      only_guess( 0, b.left() ); add_guess ( 'r', b.left() + col );
+      add_guess( 'v', b.right() ); return;
       }
     }
   if( code ) { add_guess( code, 0 ); return; }
   if( f.bp.minima( b.height() / 8 ) == 2 )
     {
     code = f.test_hknwx();
-    if( code == 'n' )	// Looks for '"' or merged 'rr' or 'rt' or 'fl'
+    if( code == 'n' )	// Looks for '"' or merged 'rr' or 'TT' or 'rt' or 'fl'
       {
       if( b.bottom() <= charbox.vcenter() ) { add_guess( '"', 0 ); return; }
       if( b.width() > b.height() && 10 * f.lp[f.lp.pos(10)] < b.width() &&
           !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
-        const int col = b.seek_left( b.vcenter(), b.right() );
-        if( col < b.hpos( 90 ) && !b.escape_top( b.vcenter(), col ) )
+        const int rgap = f.rp[f.rp.pos(50)];
+        if( 10 * rgap > b.width() && !b.escape_top( b.vcenter(), b.right() ) )
           {
           only_guess( 0, b.left() );
-          add_guess( 'r', b.hcenter() ); add_guess( 'r', b.right() );
+          const int lgap = f.lp[f.lp.pos(50)];
+          if( 10 * lgap > b.width() && !b.escape_top( b.vcenter(), b.left() ) &&
+              Ocrad::similar( lgap, rgap, 10 ) && b.top() < charbox.top() )
+            { add_guess( 'T', b.hcenter() ); add_guess( 'T', b.right() ); }
+          else { add_guess( 'r', b.hcenter() ); add_guess( 'r', b.right() ); }
           return;
           }
         }
@@ -124,7 +139,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
       if( f.rp.minima() == 1 && !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
         int dmax = 0; bool bar = false;
-        for( int row = b.vcenter(); row > b.vpos( 25 ); --row )
+        for( int row = b.vpos( 60 ); row > b.vpos( 25 ); --row )
           {
           int d = b.hcenter() - b.seek_left( row, b.hcenter() );
           if( d > dmax ) dmax = d;
@@ -179,7 +194,8 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
     }
   if( f.bp.minima() == 3 )
     {
-    if( f.tp.minima() == 3 && f.lp.minima() == 2 && f.rp.minima() == 2 )
+    if( f.bp.minima( b.height() / 2 ) == 1 && f.tp.minima() == 3 &&
+        f.lp.minima() == 2 && f.rp.minima() == 2 )
       { add_guess( '*', 0 ); return; }
     if( b.id( b.vcenter(), b.hcenter() ) == 0 &&
         b.seek_left( b.vcenter(), b.hcenter() ) <= b.hpos( 25 ) )
@@ -192,7 +208,8 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         { only_guess( 0, b.left() ); add_guess( 'r', col );
         add_guess( 'n', b.right() ); return; }
       }
-    add_guess( 'm', 0 ); return;
+    if( f.tp.minima( b.height() / 3 ) == 1 ) add_guess( 'm', 0 );
+    return;
     }
   if( f.bp.minima() == 4 && f.tp.minima( b.height() / 3 ) == 1 )
     {					// Found merged 'rm'
@@ -207,7 +224,10 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
 
   if( f.tp.minima( b.height() / 4 ) == 3 )
     {
-    int hdiff; if( !b.bottom_hook( &hdiff ) ) add_guess( 'w', 0 );
+    int hdiff;
+    if( !b.bottom_hook( &hdiff ) &&
+        ( f.vscan()[b.vcenter() - b.top()] < 4 || !b.escape_top( b.vcenter(), b.hcenter() ) ) )
+      add_guess( 'w', 0 );
     return;
     }
 
@@ -226,13 +246,13 @@ void Character::recognize111( const Charset & charset, const Rectangle & charbox
   {
   const Block & b = block( 0 );
   const Bitmap & h = b.hole( 0 );
-  if( !h.is_hcentered_in( b ) ) return;
+  if( !h.is_hcentred_in( b ) ) return;
   Features f( b );
   int top_delta = h.top() - b.top(), bottom_delta = b.bottom() - h.bottom();
 
   if( std::abs( top_delta - bottom_delta ) <= std::max( 2, h.height() / 4 ) ||
       Ocrad::similar( top_delta, bottom_delta, 40, 2 ) )
-    {					// hole is vertically centered
+    {					// hole is vertically centred
     int code = f.test_4ADQao( charset, charbox );
     if( code )
       {
@@ -278,18 +298,33 @@ void Character::recognize112( const Rectangle & charbox ) throw()
   const Block & b = block( 0 );
   const Bitmap & h1 = b.hole( 0 );		// upper hole
   const Bitmap & h2 = b.hole( 1 );		// lower hole
-  if( !h1.is_hcentered_in( b ) ) return;
-  if( !h2.is_hcentered_in( b ) ) return;
+  Profile lp( b, Profile::left );
+  Profile tp( b, Profile::top );
+  Profile rp( b, Profile::right );
+  Profile bp( b, Profile::bottom );
+
+  // Check for 'm' or 'w' with merged serifs
+  if( 10 * std::abs( h2.vcenter() - h1.vcenter() ) <= b.height() &&
+      h1.is_vcentred_in( b ) && h2.is_vcentred_in( b ) )
+    {
+    if( ( b.bottom() - h1.bottom() <= h1.top() - b.top() ) &&
+        ( b.bottom() - h2.bottom() <= h2.top() - b.top() ) && bp.isflats() )
+      { add_guess( 'm', 0 ); return; }
+
+    if( 5 * std::abs( h1.bottom() - b.vcenter() ) <= b.height() &&
+        5 * std::abs( h2.bottom() - b.vcenter() ) <= b.height() &&
+        tp.isflats() && bp.minima() == 2 )
+      { add_guess( 'w', 0 ); return; }
+    return;
+    }
+  if( !h1.is_hcentred_in( b ) ) return;
+  if( !h2.is_hcentred_in( b ) ) return;
   if( h1.left() > b.hcenter() && h2.left() > b.hcenter() ) return;
   if( h1.right() < b.hpos( 40 ) && h2.right() < b.hpos( 40 ) ) return;
   if( h1.top() > b.vcenter() || h2.bottom() < b.vcenter() ) return;
   const int a1 = h1.area();
   const int a2 = h2.area();
 
-  Profile lp( b, Profile::left );
-  Profile tp( b, Profile::top );
-  Profile rp( b, Profile::right );
-  Profile bp( b, Profile::bottom );
   if( Ocrad::similar( a1, a2, 50 ) )		// I don't like this
     {
     if( h1.bottom() > b.vcenter() && h2.top() < b.vcenter() &&
@@ -302,17 +337,17 @@ void Character::recognize112( const Rectangle & charbox ) throw()
         if( b.top_hook( &hdiff ) && hdiff > b.height() / 2 )
           { add_guess( 's', 0 ); return; }
     
-      if( lp.isflats() && !lp.ispit() ) { add_guess( 'B', 0 ); return; }
-
       for( int i = h1.bottom() - b.top() + 1; i < h2.top() - b.top(); ++i )
         if( 2 * rp[i] > b.width() ) { add_guess( 'g', 2 ); return; }
 
+      if( lp.isflats() && !lp.ispit() ) { add_guess( 'B', 0 ); return; }
+
       int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
       int col2 = h2.seek_right( h2.top(), h2.left() - 1 ) + 1;
       if( col1 <= col2 )
         {
         if( lp.isconvex() || lp.ispit() ) add_guess( 'e', 1 );
-        else if( !rp.isctip() ) add_guess( 'a', 1 );
+        else if( !rp.isctip() && tp.minima() == 1 ) add_guess( 'a', 1 );
         if( bp.istpit() ) { add_guess( '$', 0 ); return; }
         }
 
@@ -357,22 +392,34 @@ void Character::recognize112( const Rectangle & charbox ) throw()
 
   if( a1 > a2 && h1.h_overlaps( h2 ) )
     {
-    if( !h1.v_overlaps( h2 ) ) { add_guess( 'g', 0 ); return; }
+    if( !h1.v_overlaps( h2 ) )
+      {
+      if( h2.left() > b.hcenter() && h2.bottom() < b.bottom() - h1.height() )
+        add_guess( '9', 0 );
+      else add_guess( 'g', 0 );
+      return;
+      }
     if( h1.h_includes( h2 ) ) { add_guess( 'Q', 0 ); return; }
     return;
     }
 
   if( a1 < a2 && tp.minima() == 1 )
     {
-    if( h1.h_overlaps( h2 ) && rp.minima() == 1 )
+    if( h1.h_overlaps( h2 ) )
       {
-      if( 2 * h1.height() > h2.height() && 2 * h1.width() > h2.width() &&
-          3 * h2.width() >= b.width() && !lp.isctip() )
-        { if( lp.ispit() && lp.isconvex() ) add_guess( '6', 0 );
-        else add_guess( 'B', 0 ); }
-      else if( h2.right() < b.hcenter() ) add_guess( '&', 0 );
-      else add_guess( 'a', 0 );
-      return;
+      if( rp.minima() == 1 )
+        {
+        if( 2 * h1.height() > h2.height() && 2 * h1.width() > h2.width() &&
+            3 * h2.width() >= b.width() && !lp.isctip() )
+          { if( lp.ispit() && lp.isconvex() ) add_guess( '6', 0 );
+          else add_guess( 'B', 0 ); }
+        else if( h2.right() < b.hcenter() ) add_guess( '&', 0 );
+        else add_guess( 'a', 0 );
+        return;
+        }
+      if( !h1.v_overlaps( h2 ) &&
+          h1.right() < b.hcenter() && h1.top() > b.top() + h1.height() )
+        { add_guess( '6', 0 ); return; }
       }
     if( h1.bottom() < h2.top() ) { add_guess( '&', 0 ); return; }
     }
