@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
@@ -28,63 +29,65 @@
 #include "feats.h"
 
 
-// Looks for three black sections in column hcenter()  1, then tests if
+// Looks for three black sections in column hcenter()  n, then tests if
 // upper and lower gaps are open to the right or to the left
 //
 int Features::test_235Esz( const Charset & charset ) const throw()
   {
-  static const int csize = 9;
-  static const int ucoff[csize] = { 0, +1,  0, -1,  0, +1, +1, -1, -1 };
-  static const int lcoff[csize] = { 0,  0, -1,  0, +1, -1, +1, -1, +1 };
+  static const int csize = 3;
+  static const int ucoff[csize] = { 0, -1, +1 };
+  static const int lcoff[3*csize] = { 0, -1, +1, -1, 0, +1, +1, 0, -1 };
   const Block & b = *_block;
-  if( b.width() < 9 || b.height() > 3 * b.width() ) return 0;
+  if( b.width() < 9 || b.height() > 3 * b.width() ||
+      bp.minima( b.height() / 2 ) > 1 ) return 0;
 
-  const bool cid = b.get_bit( b.vcenter(), b.hcenter() );
   const int noise = ( std::min( b.height(), b.width() ) / 15 ) + 1;
   int lrow1 = 0, urow2 = 0, lrow2 = 0, urow3 = 0;
-  int lcol1 = 0, ucol2 = 0, lcol2 = 0, ucol3 = 0, black_section = 0;
+  int lcol1 = 0, ucol2 = 0, lcol2 = 0, ucol3 = 0;
+  bool done = false;
 
-  for( int i = 0; i < csize; ++i )
+  for( int i = 0; i < csize && !done; ++i )
     {
-    int ucol = b.hcenter() + ( noise * ucoff[i] );
-    int lcol = b.hcenter() + ( noise * lcoff[i] );
-    if( ucol != lcol )
+    const int ucol = b.hcenter() + ( noise * ucoff[i] );
+    int row = b.top() + tp[ucol-b.left()];
+    while( ++row < b.bottom() && b.get_bit( row, ucol ) ) ;
+    if( row <= b.vpos( 30 ) ) { lrow1 = row; lcol1 = ucol; } else continue;
+    while( ++row < b.bottom() && !b.get_bit( row, ucol ) ) ;
+    if( row < b.bottom() )
       {
-      const int l = std::min( ucol, lcol ), r = std::max( ucol, lcol );
-      int j;
-      for( j = l; j <= r; ++j ) if( b.get_bit( b.vcenter(), j ) != cid ) break;
-      if( j <= r ) continue;
+      urow2 = row - 1; ucol2 = ucol;
+      for( int j = 0; j < csize && !done; ++j )
+        {
+        row = urow2 + 1;
+        const int lcol = b.hcenter() + ( noise * lcoff[(csize*i)+j] );
+        if( ucol != lcol )
+          {
+          const int d = ( ucol > lcol ) ? +1 : -1;
+          int c = lcol; while( c != ucol && b.get_bit( row, c ) ) c += d;
+          if( c != ucol ) continue;
+          }
+        while( ++row < b.bottom() && b.get_bit( row, lcol ) ) ;
+        if( row < b.bottom() ) { lrow2 = row; lcol2 = lcol; } else continue;
+        while( ++row <= b.bottom() && !b.get_bit( row, lcol ) ) ;
+        if( row <= b.bottom() && row > b.vpos( 70 ) )
+          { urow3 = row - 1; ucol3 = lcol; done = true; }
+        }
       }
-    bool prev_black = false;
-    for( int row = b.top(); row <= b.bottom(); ++row )
-      {
-      int col = ( row <= b.vcenter() ) ? ucol : lcol;
-      bool black = b.get_bit( row, col );
-      if( black )
-        { if( !prev_black )
-          { if( ++black_section == 2 ) { urow2 = row - 1; ucol2 = col; }
-          else if( black_section == 3 ) { urow3 = row - 1; ucol3 = col; } } }
-      else if( prev_black )
-        { if( black_section == 1 ) { lrow1 = row; lcol1 = col; }
-          else if( black_section == 2 ) { lrow2 = row; lcol2 = col; } }
-      prev_black = black;
-      }
-    if( black_section == 3 && lrow1 <= b.vpos( 30 ) && urow3 >= b.vpos( 70 ) )
-      break;
-    black_section = 0;
     }
+  if( !done ) return 0;
 
-  if( black_section != 3 ) return 0;
-  bool tbopen = b.escape_bottom( urow3, ucol3 ) && b.escape_top( lrow1, lcol1 );
-  const int ascode = cid ? '*' : 0;
+  const bool bopen = b.escape_bottom( urow3, ucol3 );
+  const bool topen = b.escape_top( lrow1, lcol1 );
+  const bool tbopen = bopen && topen;
+  const int ascode = ( b.get_bit( b.vcenter(), b.hcenter() ) ) ? '*' : 0;
   if( b.escape_left( lrow2, lcol2 ) )
     {
     if( b.escape_left( urow2, ucol2 ) )
       {
       if( tbopen ) return ascode;
-      if( b.height() <= 3 * b.width() )
+      if( !bopen && !topen && b.height() <= 3 * b.width() )
         {
-        int lm = lp.minima(), rm = rp.minima();
+        const int lm = lp.minima(), rm = rp.minima();
         if( ( lm == 3 || lm == 2 ) &&
             ( rm == 2 || ( rm == 1 && rp.iminimum() < rp.pos( 80 ) ) ) )
           return '3';
@@ -93,8 +96,9 @@ int Features::test_235Esz( const Charset & charset ) const throw()
     else if( b.escape_right( urow2, ucol2 ) )
       {
       if( tbopen ) return ascode;
-      if( b.seek_right( lrow1 + 1, lcol1 ) >= b.right() &&
-          b.seek_left( urow3 - 1, ucol3 ) > b.left() )
+      if( rp[lrow1 + 1 - b.top()] >= lcol1 - b.left() &&
+          ( lp[lrow2 + 1 - b.top()] < lcol2 - b.left() ||
+            lp[urow3 - 1 - b.top()] < ucol3 - b.left() ) )
         {
         int hdiff;
         if( !b.top_hook( &hdiff ) || 5 * hdiff >= 4 * b.height() )
@@ -119,7 +123,7 @@ int Features::test_235Esz( const Charset & charset ) const throw()
         if( 8 * lp[((lrow2+urow3)/2)-b.top()] >= b.width() &&
             b.escape_top( ( lrow1 + urow2 ) / 2, b.left() ) &&
             !b.escape_top( ( lrow2 + urow3 ) / 2, b.left() ) ) return 'f';
-        if( b.escape_bottom( urow3, ucol3 ) )
+        if( rp.minima( b.width() / 8 ) < 3 && b.escape_bottom( urow3, ucol3 ) )
           {
           if( charset.enabled( Charset::iso_8859_15 ) ||
               charset.enabled( Charset::iso_8859_9 ) )
@@ -198,7 +202,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
           vbar(0).h_overlaps( hbar(1) ) )
         {
         if( Ocrad::similar( hbar(0).width(), hbar(2).width(), 10 ) &&
-            hbar(2).width() >= hbar(1).width() &&
+            10 * hbar(2).width() >= 9 * hbar(1).width() &&
             hbar(0).left() <= hbar(1).left() + 1 )
           return 'E';
         if( hbar(0).width() > hbar(2).width() &&
@@ -348,24 +352,26 @@ int Features::test_c() const throw()
   }
 
 
-int Features::test_frst() const throw()
+int Features::test_frst( const Rectangle & charbox ) const throw()
   {
   const Block & b = *_block;
 
-  if( tp.minima( b.height() / 4 ) != 1 || bp.minima( b.height() / 4 ) != 1 ||
-      tp.minima( b.height() / 2 ) != 1 || bp.minima( b.height() / 2 ) != 1 )
-    return 0;
+  if( bp.minima( b.height() / 4 ) != 1 || tp.minima( b.height() / 2 ) != 1 ||
+      bp.minima( b.height() / 2 ) != 1 ) return 0;
+  const bool maybe_slanted_r = ( tp.minima( b.height() / 4 ) != 1 );
+  bool maybe_t = true;
 
+  if( !maybe_slanted_r )
     {
     int b_hdiff = 0, t_hdiff = 0;
     if( b.bottom_hook( &b_hdiff ) )
       {
-      if( -b_hdiff > b.height() / 2 )
+      if( -2 * b_hdiff > b.height() )
         {
         if( b.height() >= 3 * wp.max() &&
             ( hbars() == 0 || hbar(0).bottom() < b.vpos( 20 ) ) ) return 'l';
-        if( 2 * wp[wp.pos(5)] < b.width() && hbars() >= 1 && hbars() <= 2 &&
-            hbar(0).top() >= b.vpos( 20 ) && hbar(0).bottom() < b.vcenter() &&
+        if( 2 * wp[wp.pos(6)] < b.width() && hbars() >= 1 && hbars() <= 2 &&
+            hbar(0).top() >= b.vpos( 15 ) && hbar(0).bottom() < b.vcenter() &&
             Ocrad::similar( hbar(0).width(), wp.max(), 10 ) ) return 't';
         }
       }
@@ -377,22 +383,24 @@ int Features::test_frst() const throw()
         return 'f';
       if( 2 * b_hdiff > b.height() && 2 * t_hdiff > b.height() )
         return 0;	// recognized 's' or SCCEDI
+      maybe_t = false;
       }
     }
 
-  if( !vbars() && 2 * rp[rp.pos(50)] > b.width() &&
+  if( 2 * rp[rp.pos(50)] > b.width() &&
       2 * bp[bp.pos(50)] > b.height() && tp.isctip() ) return 'r';
 
-  if( vbars() != 1 || vbar(0).width() < 2 ) return 0;
+  if( maybe_slanted_r || vbars() != 1 || vbar(0).width() < 2 ) return 0;
   if( vbar(0).hcenter() <= b.hcenter() )
     {
-    const int col = b.right() - rp[b.vcenter()-b.top()] + 2;
+    const int col = b.right() - rp[rp.pos(50)] + 2;
     if( col < b.right() )
       {
       const int row = b.seek_bottom( b.vcenter(), col );
       if( row >= b.bottom() || b.escape_bottom( row - 1, col ) )
         {
-        if( rp.minima() == 3 ) return 'f';
+        if( rp.minima() == 3 )
+          { if( rp.minima( b.width() / 8 ) < 3 ) return 'f'; else return 0; }
         if( Ocrad::similar( b.height(), b.width(), 40 ) )
           {
           if( tp.minima( b.height() / 8 ) == 2 &&
@@ -414,32 +422,33 @@ int Features::test_frst() const throw()
             }
           }
         }
-      if( Ocrad::similar( b.height(), b.width(), 40 ) )
-        {
-        if( vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
-            b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
-          return 'x';
-        if( lp.istpit() ) return 't';
-        }
+      if( Ocrad::similar( b.height(), b.width(), 40 ) &&
+          vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
+          b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
+        return 'x';
       }
-    if( 3 * b.height() >= 5 * b.width() && vbar(0).left() > b.left() &&
-             rp.minima() <= 2 )
+    if( 3 * b.height() > 4 * b.width() && vbar(0).left() > b.left() &&
+        rp.minima() <= 2 )
       {
-      const int col = b.right() - std::max( 0, rp[b.vcenter() - b.top()] - 1 );
+      const int col = b.right() - std::max( 0, rp[rp.pos(50)] - 1 );
       if( !b.escape_bottom( b.vcenter(), col ) )
-        { if( tp.ispit() && lp.iminimum() < lp.pos( 40 ) ) return 't'; }
+        { if( 3 * wp[wp.pos(6)] < 2 * b.width() && tp.ispit() && lp.iminimum() < lp.pos( 40 ) ) return 't'; }
       else if( 2 * wp.max() > b.width() )
         {
         if( rp.iminimum() < rp.pos( 20 ) )
           {
           if( rp.increasing( rp.pos( 20 ) ) || bp.increasing() ||
               tp.minima( 1 ) == 2 ||
-              ( rp.minima() == 1 && tp.iminimum() > tp.pos( 50 ) ) ) return 'r';
-          else return 'f';
+              ( rp.minima() == 1 && ( b.height() < charbox.height() || tp.iminimum() > tp.pos( 50 ) ) ) )
+            return 'r';
+          else if( 3 * b.height() >= 5 * b.width() ) return 'f';
           }
         else
-          { if( !rp.isconvex() && bp.minima( b.height() / 3 ) == 1 ) return 't';
-          else return 0; }
+          {
+          if( maybe_t && !rp.isconvex() && bp.minima( b.height() / 3 ) == 1 )
+            return 't';
+          else return 0;
+          }
         }
       }
     if( b.seek_bottom( b.vcenter(), b.hpos( 60 ) + 1 ) >= b.bottom() )
@@ -525,10 +534,10 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
       const int lc = ( lg + ( 2 * ( lp.limit() - rg ) ) ) / 3;
       const int lc2 = ( lg2 + lp.limit() - rg2 ) / 2;
       if( bp.ispit() && 7 * tp.range() < 4 * b.height() &&
-          2 * wp[wp.pos(70)] < b.width() )
-        if( ( Ocrad::similar( lg, rg, 20 ) &&
-              5 * wp[wp.pos(98)] >= 4 * wp[wp.pos(70)] ) ||
-            ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) return 'Y';
+          2 * wp[wp.pos(70)] < b.width() &&
+          ( ( Ocrad::similar( lg, rg, 20 ) &&
+            5 * wp[wp.pos(98)] >= 4 * wp[wp.pos(70)] ) ||
+            ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) ) return 'Y';
       if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() )
         return 'u';
       if( lg < rg + 1 && !lp.increasing( lp.pos( 50 ) ) &&
@@ -563,7 +572,14 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
             tp.minima( b.height() / 2 ) <= 2 &&
             tp.minima( ( 2 * b.height() ) / 5 ) <= 2 && !lp.istpit() &&
             4 * std::abs( rp[rp.pos(20)] - rp[rp.pos(80)] ) <= b.width() )
-          return 'N';
+          {
+          Rectangle r( b.left(), b.top(), b.hcenter(), b.bottom() );
+          Bitmap bm( b, r );
+          int hdiff;
+          if( bm.seek_bottom( bm.top(), bm.right() ) > bm.vcenter() &&
+              bm.bottom_hook( &hdiff ) && -2 * hdiff > bm.height() ) return 'u';
+          else return 'N';
+          }
         return 0;
         }
       if( 3 * lg < 2 * rg && lg < b.width() / 4 && rg > b.width() / 4 &&
@@ -584,7 +600,7 @@ int Features::test_hknwx() const throw()
   {
   const Block & b = *_block;
 
-  if( tp.minima( b.height() / 8 ) == 2 &&
+  if( tp.minima( b.height() / 8 ) == 2 && bp.minima( b.height() / 2 ) == 1 &&
       ( ( lp.istip() && rp.istip() ) ||
         ( lp.isconcave() && rp.isconcave() ) ) ) return 'x';
 
