@@ -28,8 +28,8 @@
 #include <cstring>
 #include <string>
 #include <vector>
-#include <getopt.h>
 
+#include "arg_parser.h"
 #include "common.h"
 #include "rational.h"
 #include "rectangle.h"
@@ -39,12 +39,12 @@
 
 namespace {
 
-// Date of this version: 2006-02-15
+// Date of this version: 2006-04-03
 
 const char * invocation_name = 0;
 const char * const Program_name    = "GNU Ocrad";
 const char * const program_name    = "ocrad";
-const char * const program_version = "0.14";
+const char * const program_version = "0.15";
 const char * const program_year    = "2006";
 
 struct Input_control
@@ -227,9 +227,8 @@ bool Ocrad::verbose = false;
 
 void Ocrad::internal_error( const char * msg ) throw()
   {
-  char buf[80];
-  std::snprintf( buf, sizeof( buf ), "internal error: %s.\n", msg );
-  show_error( buf );
+  std::string s( "internal error: " ); s += msg;
+  show_error( s.c_str() );
   exit( 3 );
   }
 
@@ -248,71 +247,80 @@ int main( int argc, char * argv[] ) throw()
   bool append = false, force = false;
   invocation_name = argv[0];
 
-  // scan arguments
-
-  while( true )			// process options
+  static const Arg_parser::Option options[] =
     {
-    static const struct option long_options[] =
-      {
-      {"append",          no_argument, 0, 'a'},
-      {"block",     required_argument, 0, 'b'},
-      {"charset",   required_argument, 0, 'c'},
-      {"copy",            no_argument, 0, 'C'},
-      {"crop",      required_argument, 0, 'p'},
-      {"debug",     required_argument, 0, 'D'},
-      {"force",           no_argument, 0, 'f'},
-      {"format",    required_argument, 0, 'F'},
-      {"help",            no_argument, 0, 'h'},
-      {"invert",          no_argument, 0, 'i'},
-      {"layout",    required_argument, 0, 'l'},
-      {"scale",     required_argument, 0, 's'},
-      {"transform", required_argument, 0, 't'},
-      {"threshold", required_argument, 0, 'T'},
-      {"verbose",         no_argument, 0, 'v'},
-      {"version",         no_argument, 0, 'V'},
-      {0, 0, 0, 0}
-      };
+    { '1', 0,           Arg_parser::no  },
+    { '2', 0,           Arg_parser::no  },
+    { '3', 0,           Arg_parser::no  },
+    { '4', 0,           Arg_parser::no  },
+    { '5', 0,           Arg_parser::no  },
+    { '6', 0,           Arg_parser::no  },
+    { 'a', "append",    Arg_parser::no  },
+    { 'b', "block",     Arg_parser::yes },
+    { 'c', "charset",   Arg_parser::yes },
+    { 'C', "copy",      Arg_parser::no  },
+    { 'D', "debug",     Arg_parser::yes },
+    { 'f', "force",     Arg_parser::no  },
+    { 'F', "format",    Arg_parser::yes },
+    { 'h', "help",      Arg_parser::no  },
+    { 'i', "invert",    Arg_parser::no  },
+    { 'l', "layout",    Arg_parser::yes },
+    { 'o', 0,           Arg_parser::yes },
+    { 'p', "crop",      Arg_parser::yes },
+    { 's', "scale",     Arg_parser::yes },
+    { 'S', 0,           Arg_parser::yes },
+    { 't', "transform", Arg_parser::yes },
+    { 'T', "threshold", Arg_parser::yes },
+    { 'v', "verbose",   Arg_parser::no  },
+    { 'V', "version",   Arg_parser::no  },
+    { 'x', 0,           Arg_parser::yes },
+    {  0 , 0,           Arg_parser::no  } };
 
-    int c = getopt_long( argc, argv, "123456CD:F:S:T:Vab:c:fhil:o:p:s:t:vx:",
-                         long_options, 0 );
-    if( c == -1 ) break;		// all options processed
+  Arg_parser parser( argc, argv, options );
+  if( parser.error().size() )				// bad option
+    { show_error( parser.error().c_str(), 0, true ); return 1; }
 
-    switch( c )
+  int argind;
+  for( argind = 0; argind < parser.arguments(); ++argind )
+    {
+    const int code = parser.code( argind );
+    if( !code ) break;					// no more options
+    const char * arg = parser.argument( argind ).c_str();
+    switch( code )
       {
       case '1':
       case '2':
       case '3':
       case '4':
       case '5':
-      case '6': control.filetype = c; break;
+      case '6': control.filetype = code; break;
       case 'C': input_control.copy = true; break;
-      case 'D': control.debug_level = std::strtol( optarg, 0, 0 ); break;
-      case 'F': if( !control.set_format( optarg ) )
+      case 'D': control.debug_level = std::strtol( arg, 0, 0 ); break;
+      case 'F': if( !control.set_format( arg ) )
                   { show_error( "bad output format", 0, true ); return 1; }
                 break;
-      case 'S': input_control.specialtype = optarg[0]; break;
-      case 'T': if( !input_control.set_threshold( optarg ) ) return 1; break;
+      case 'S': input_control.specialtype = arg[0]; break;
+      case 'T': if( !input_control.set_threshold( arg ) ) return 1; break;
       case 'V':	show_version(); return 0;
       case 'a': append = true; break;
-      case 'b': input_control.rindex = std::strtol( optarg, 0, 0 ) - 1; break;
-      case 'c': if( !control.charset.enable( optarg ) )
-                  { control.charset.show_error( program_name, optarg ); return 1; }
+      case 'b': input_control.rindex = std::strtol( arg, 0, 0 ) - 1; break;
+      case 'c': if( !control.charset.enable( arg ) )
+                  { control.charset.show_error( program_name, arg ); return 1; }
                 break;
       case 'f': force = true; break;
       case 'h': show_help(); return 0;
       case 'i': input_control.invert = true; break;
-      case 'l': input_control.layout_level = std::strtol( optarg, 0, 0 ); break;
-      case 'o':	outfile_name = optarg; break;
-      case 'p': if( !input_control.read_crop_rectangle( optarg ) ) return 1; break;
-      case 's': input_control.scale = std::strtol( optarg, 0, 0 ); break;
-      case 't': if( !input_control.transformation.set( optarg ) )
-                  { input_control.transformation.show_error( program_name, optarg );
+      case 'l': input_control.layout_level = std::strtol( arg, 0, 0 ); break;
+      case 'o':	outfile_name = arg; break;
+      case 'p': if( !input_control.read_crop_rectangle( arg ) ) return 1; break;
+      case 's': input_control.scale = std::strtol( arg, 0, 0 ); break;
+      case 't': if( !input_control.transformation.set( arg ) )
+                  { input_control.transformation.show_error( program_name, arg );
                   return 1; }
                 break;
       case 'v': Ocrad::verbose = true; break;
-      case 'x':	exportfile_name = optarg; break;
-      case '?': show_error( 0, 0, true ); return 1;	// bad option
-      default: return 1;
+      case 'x':	exportfile_name = arg; break;
+      default : Ocrad::internal_error( "uncaught option" );
       }
     } // end process options
 
@@ -350,7 +358,7 @@ int main( int argc, char * argv[] ) throw()
     }
 
   // process any remaining command line arguments (input files)
-  FILE *infile = (optind < argc) ? 0 : stdin;
+  FILE *infile = (argind < parser.arguments()) ? 0 : stdin;
   const char *infile_name = "-";
   int retval = 0;
   while( true )
@@ -358,8 +366,8 @@ int main( int argc, char * argv[] ) throw()
     while( infile != stdin )
       {
       if( infile ) std::fclose( infile );
-      if( optind >= argc ) { infile = 0; break; }
-      infile_name = argv[optind++];
+      if( argind >= parser.arguments() ) { infile = 0; break; }
+      infile_name = parser.argument( argind++ ).c_str();
       if( std::strcmp( infile_name, "-" ) == 0 ) infile = stdin;
       else infile = std::fopen( infile_name, "r" );
       if( infile ) break;
