@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,6 @@ public:
   void print( const Control & control ) const throw();
   void dprint( const Control & control, bool graph, bool recursive ) const throw();
   void xprint( const Control & control ) const throw();
-  void cmark( Bitmap & bitmap ) const throw();
-  void lmark( Bitmap & bitmap ) const throw();
+  void cmark( Page_image & page_image ) const throw();
+  void lmark( Page_image & page_image ) const throw();
   };
