@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,24 +16,27 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
+#include <algorithm>
 #include <cctype>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
+#include "rational.h"
 #include "rectangle.h"
-#include "vrhomboid.h"
 #include "track.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "block.h"
 #include "character.h"
+#include "page_image.h"
 #include "textline.h"
 
 
 Textline::Textline( const Textline & tl ) throw()
-  : Track( tl )
+  : Track( tl ), _big_initial( 0 )
   {
-  big_initial( tl._big_initial );
+  if( tl._big_initial ) big_initial( new Character( *tl._big_initial ) );
   for( unsigned int i = 0; i < tl.cpv.size(); ++i )
     shift_characterp( new Character( *tl.cpv[i] ) );
   }
@@ -44,7 +47,8 @@ Textline & Textline::operator=( const Textline & tl ) throw()
   if( this != &tl )
     {
     Track::operator=( tl );
-    big_initial( tl._big_initial );
+    if( tl._big_initial ) big_initial( new Character( *tl._big_initial ) );
+    else big_initial( 0 );
     for( unsigned int i = 0; i < cpv.size(); ++i ) delete cpv[i];
     cpv.clear();
     for( unsigned int i = 0; i < tl.cpv.size(); ++i )
@@ -125,10 +129,8 @@ bool Textline::insert_space( int i, bool tab ) throw()
 void Textline::join( Textline & tl ) throw()
   {
   if( tl._big_initial &&
-      ( !_big_initial || tl._big_initial->h_precedes( *_big_initial ) ) )
-    {
-    delete _big_initial; _big_initial = tl._big_initial; tl._big_initial = 0;
-    }
+      ( !_big_initial || tl._big_initial->left() < _big_initial->left() ) )
+    { big_initial( tl._big_initial ); tl._big_initial = 0; }
 
   for( int i = 0; i < tl.characters(); ++i ) shift_characterp( tl.cpv[i] );
 
@@ -161,28 +163,28 @@ int Textline::mean_height() const throw()
   }
 
 
-int Textline::mean_width() const throw()
+Rational Textline::mean_width() const throw()
   {
   int c = 0, sum = 0;
 
   for( int i = 0; i < characters(); ++i )
     if( !cpv[i]->maybe(' ') ) { ++c; sum += cpv[i]->width(); }
-  if( c ) sum /= c;
-  return sum;
+
+  if( c ) return Rational( sum, c );
+  return 0;
   }
 
 
-int Textline::mean_gap_width( int max_gap_width ) const throw()
+Rational Textline::mean_gap_width( const int first, int last ) const throw()
   {
-  int gaps = characters() - 1, sum = 0;
+  if( last < 0 ) last = characters() - 1;
+  int sum = 0;
 
-  for( int i = 0; i < gaps; ++i )
-    {
-    int gap = cpv[i+1]->left() - cpv[i]->right() - 1;
-    if( gap > 0 && ( !max_gap_width || gap <= max_gap_width ) ) sum += gap;
-    }
-  if( gaps > 0 ) sum /= gaps;
-  return sum;
+  for( int i = first; i < last; ++i )
+    sum += std::max( 0, cpv[i+1]->left() - cpv[i]->right() - 1 );
+
+  if( last > first ) return Rational( sum, last - first );
+  return 0;
   }
 
 
@@ -243,11 +245,11 @@ void Textline::xprint( const Control & control ) const throw()
   }
 
 
-void Textline::cmark( Bitmap & bitmap ) const throw()
+void Textline::cmark( Page_image & page_image ) const throw()
   {
-  if( _big_initial ) bitmap.draw_rectangle( *_big_initial );
+  if( _big_initial ) page_image.draw_rectangle( *_big_initial );
   for( int i = 0; i < characters(); ++i )
-    bitmap.draw_rectangle( character(i) );
+    page_image.draw_rectangle( character(i) );
   }
 
 
