@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-class Bitmap;
+class Page_image;
+class Rational;
 
 class Textline : public Track
   {
@@ -44,15 +45,15 @@ public:
   int characters() const throw() { return cpv.size(); }
 
   int mean_height() const throw();
-  int mean_width() const throw();
-  int mean_gap_width( int max_gap_width = 0 ) const throw();
+  Rational mean_width() const throw();
+  Rational mean_gap_width( const int first = 0, int last = -1 ) const throw();
   int mean_hcenter() const throw();
   int mean_vcenter() const throw();
 
   void print( const Control & control ) const throw();
   void dprint( const Control & control, bool graph, bool recursive ) const throw();
   void xprint( const Control & control ) const throw();
-  void cmark( Bitmap & bitmap ) const throw();
+  void cmark( Page_image & page_image ) const throw();
 
   void recognize1( const Charset & charset ) const throw();
   void recognize2( const Charset & charset ) throw();
