@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,9 @@
 #include <cctype>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
-#include "vrhomboid.h"
 #include "track.h"
 #include "ucs.h"
 #include "bitmap.h"
@@ -34,9 +34,9 @@
 
 namespace {
 
-int find_space( const std::vector< Character * > & cpv, int i ) throw()
+int find_space_or_hyphen( const std::vector< Character * > & cpv, int i ) throw()
   {
-  while( i < (int)cpv.size() && !cpv[i]->maybe(' ') ) ++i;
+  while( i < (int)cpv.size() && !cpv[i]->maybe(' ') && !cpv[i]->maybe('-') ) ++i;
   return i;
   }
 
@@ -47,6 +47,32 @@ void Textline::recognize2( const Charset & charset ) throw()
   {
   if( characters() == 0 ) return;
 
+  // try to recognize separately the 3 overlapped blocks of an
+  // unrecognized character
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( !c.guesses() && c.blocks() == 3 )
+      {
+      const Block & b1 = c.block( 0 );
+      const Block & b2 = c.block( 1 );
+      const Block & b3 = c.block( 2 );		// lower block
+      if( Ocrad::similar( b2.height(), b3.height(), 20 ) && !b2.h_overlaps( b3 ) &&
+          b2.v_includes( b3.vcenter() ) && b3.v_includes( b2.vcenter() ) &&
+          b1.bottom() < b2.top() && b1.bottom() < b3.top() )
+        {
+        Character c2( new Block( b2 ) );
+        Character c3( new Block( b3 ) );
+        if( b2.h_includes( b1.hcenter() ) ) c2.shift_blockp( new Block( b1 ) );
+        else if( b3.h_includes( b1.hcenter() ) ) c3.shift_blockp( new Block( b1 ) );
+        c2.recognize1( charset, charbox( c2 ) );
+        c3.recognize1( charset, charbox( c3 ) );
+        if( c2.guesses() && c3.guesses() )
+          { c = c2; shift_characterp( new Character( c3 ) ); ++i; }
+        }
+      }
+    }
+
   // try to recognize separately the 2 overlapped blocks of an
   // unrecognized character
   for( int i = 0; i < characters(); ++i )
@@ -85,28 +111,44 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
     }
 
-  // try to separate lightly merged characters
-  // FIXME try all possible separation points
+  // remove speckles above the charbox' top of an unrecognized character
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.width() > 20 && c.width() > height() &&
-        4 * c.width() >= 3 * c.height() &&
-        Ocrad::similar( c.height(), mean_height(), 40 ) )
+    if( !c.guesses() && c.blocks() == 2 &&
+        c.block( 1 ).size() > 5 * c.block( 0 ).size() &&
+        c.block( 0 ).bottom() + 2 * c.block( 0 ).height() < charbox( c ).top() )
+      {
+      Character c1( new Block( c.block( 1 ) ) );
+      c1.recognize1( charset, charbox( c1 ) );
+      if( c1.guesses() ) c = c1;
+      }
+    }
+
+  // try to separate lightly merged characters
+  // FIXME try all possible separation points
+  // FIXME sometimes leaves unconnected blocks
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( !c.guesses() && c.width() > 20 && 5 * c.width() >= 3 * c.height() &&
+        5 * c.height() >= 3 * mean_height() )
       {
       int ib = c.blocks() - 1;
       for( int k = ib - 1; k >= 0; --k )
         if( c.block( k ).width() > c.block( ib ).width() ) ib = k;
-      if( ib < 0 || 10 * c.block( ib ).width() < 9 * c.width() )
+      if( ib < 0 || 10 * c.block( ib ).width() < 9 * c.width() ||
+          c.block( ib ).bottom() < c.bottom() )
         continue;
-      const Block & b = c.block( ib );
+      const Block & b = c.block( ib );			// widest block
       int colmin = 0, cmin = b.height() + 1;
       for( int col = b.hpos( 30 ); col <= b.hpos( 70 ); ++col )
         {
         int c = 0;
         for( int row = b.top(); row <= b.bottom(); ++row )
           if( b.id( row, col ) ) ++c;
-        if( c < cmin ) { cmin = c; colmin = col; }
+        if( c < cmin || ( c == cmin && col <= b.hcenter() ) )
+          { cmin = c; colmin = col; }
         }
       if( 4 * cmin > b.height() ||
           ( 5 * cmin > b.height() &&
@@ -114,8 +156,11 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( colmin <= b.left() || colmin >= b.right() ) continue;
       Rectangle r1( b.left(), b.top(), colmin - 1, b.bottom() );
       Rectangle r2( colmin + 1, b.top(), b.right(), b.bottom() );
-      Block b1( b, r1 ); b1.adjust_height(); b1.find_holes();
-      Block b2( b, r2 ); b2.adjust_height(); b2.find_holes();
+      Block b1( b, r1 );
+      b1.adjust_height(); if( 2 * b1.height() < b.height() ) continue;
+      Block b2( b, r2 );
+      b2.adjust_height(); if( 2 * b2.height() < b.height() ) continue;
+      b1.find_holes(); b2.find_holes();
       Character c1( new Block( b1 ) );
       Character c2( new Block( b2 ) );
       for( int j = 0; j < c.blocks(); ++j ) if( j != ib )
@@ -176,7 +221,8 @@ void Textline::recognize2( const Charset & charset ) throw()
         Block & b = c.block( c.blocks() - 1 );
         Rectangle re( left, b.top(), c.guess( g ).value, b.bottom() );
         b.add_rectangle( re );
-        Character c1( new Block( b, re ) );
+        Block b1( b, re ); b1.adjust_height(); b1.find_holes();
+        Character c1( new Block( b1 ) );
         for( int k = 0; k < c.blocks() - 1; ++k )
           if( !c.block( k ).includes( re ) && re.includes_hcenter( c.block( k ) ) )
             c1.shift_blockp( new Block( c.block( k ) ) );
@@ -233,8 +279,8 @@ void Textline::recognize2( const Charset & charset ) throw()
           {
           int code2 = c2.guess(0).code;
           if( code2 >= 128 ) continue;
-          if( ( std::isalpha( code2 ) && 4 * c1.height() > 5 * c2.height() ) ||
-              ( std::isupper( code2 ) &&
+          if( ( ( std::isalpha( code2 ) || code2 == ':' ) && 4 * c1.height() > 5 * c2.height() ) ||
+              ( ( std::isdigit( code2 ) || std::isupper( code2 ) || code2 == 'l' ) &&
                 ( c1.height() >= c2.height() ||
                   Ocrad::similar( c1.height(), c2.height(), 10 ) ) ) )
             { c1.swap_guesses( 0, 1 ); break; }
@@ -252,10 +298,8 @@ void Textline::recognize2( const Charset & charset ) throw()
       {
       int code = c1.guess( 0 ).code;
       if( !UCS::islower_small_ambiguous( code ) ) continue;
-      if( 4 * c1.height() > 5 * mean_height() )
-        { c1.only_guess( std::toupper( code ), 0 ); continue; }
       if( 5 * c1.height() < 4 * mean_height() ) continue;
-      bool capital = false, small = false;
+      bool capital = ( 4 * c1.height() > 5 * mean_height() ), small = false;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
@@ -271,7 +315,7 @@ void Textline::recognize2( const Charset & charset ) throw()
                      Ocrad::similar( c1.height(), c2.height(), 10 ) ) )
             capital = true;
           }
-        if( !small && std::islower( code2 ) &&  code2 != 'l' )
+        if( !small && std::islower( code2 ) && code2 != 'j' )
           {
           if( 5 * c1.height() < 4 * c2.height() ) small = true;
           else if( UCS::islower_small( code2 ) &&
@@ -444,13 +488,13 @@ void Textline::recognize2( const Charset & charset ) throw()
   // transform words like 'lO.OOO' into numbers like '10.000'
   for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
     {
-    end = find_space( cpv, begin );
+    end = find_space_or_hyphen( cpv, begin );
     if( end - begin < 2 ) continue;
     Character & c1 = character( begin );
     if( !c1.guesses() ) continue;
     int height = c1.height();
     int code = c1.guess( 0 ).code;
-    if( code == 'l' || code == 'O' || code == 'o' )
+    if( std::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' )
       {
       int digits = 1;
       int i = begin + 1;
@@ -460,10 +504,11 @@ void Textline::recognize2( const Charset & charset ) throw()
         if( !c.guesses() ) break;
         bool valid = false;
         int code = c.guess(0).code;
-        if( ( code == 'l' || code == 'O' || code == 'o' ) &&
+        if( ( std::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' ) &&
             Ocrad::similar( c.height(), height, 10 ) )
           { valid = true; ++digits; }
-        if( code == '.' || code == ',' ) valid = true;
+        if( code == '.' || code == ',' || code == ':' || code == '+' || code == '-' )
+          valid = true;
         if( !valid ) break;
         }
       if( i >= end && digits >= 2 )
@@ -479,6 +524,27 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
     }
 
+  // detects Roman numerals 'II', 'III' and 'IIII'
+  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
+    {
+    end = find_space_or_hyphen( cpv, begin );
+    if( end - begin < 2 || end - begin > 4 ) continue;
+    const int height = character( begin ).height();
+    int i;
+    for( i = begin; i < end; ++i )
+      {
+      Character & c = character( i );
+      if( !c.maybe('|') || !Ocrad::similar( c.height(), height, 10 ) ) break;
+      }
+    if( i >= end )
+      for( i = begin; i < end; ++i )
+        {
+        Character & c = character( i );
+        if( !character(i).maybe('I') )
+          c.insert_guess( 0, 'I', c.guess(0).value + 1 );
+        }
+    }
+
   // transform a vertical bar into 'l' or 'I' (or a 'l' into an 'I')
   for( int i = 0; i < characters(); ++i )
     {
