@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ class Textpage : public Rectangle
   std::vector< Textblock * > tbpv;
 
 public:
-  Textpage( const Bitmap & page_image, const std::vector< Rectangle > & rv,
+  Textpage( const Page_image & page_image,
             const char * filename, const Control & control ) throw();
   ~Textpage() throw();
 
